@@ -1,7 +1,7 @@
 // XGParam.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2022, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2023, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -166,6 +166,14 @@ public:
 	unsigned short note() const;
 	const char    *name() const;
 
+	// Default values accessors.
+	unsigned short level() const;
+	unsigned short group() const;
+	unsigned short pan() const;
+	unsigned short reverb() const;
+	unsigned short chorus() const;
+	unsigned short noteOff() const;
+
 private:
 
 	// Parameter descriptor.
