@@ -1,7 +1,7 @@
 // qxgeditCheck.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2019, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2023, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -54,6 +54,11 @@ signals:
 	// Value change signal.
 	void valueChanged(unsigned short);
 
+public slots:
+
+	// Value settler.
+	void setValue(unsigned short);
+
 protected slots:
 
 	// Internal widget slots.
