@@ -1,7 +1,7 @@
 // qxgeditDrumEg.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2019, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2020, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -23,6 +23,7 @@
 #include "qxgeditDrumEg.h"
 
 #include <QPainter>
+#include <QPainterPath>
 #include <QMouseEvent>
 
 
@@ -30,9 +31,8 @@
 // qxgeditDrumEg -- Custom widget
 
 // Constructor.
-qxgeditDrumEg::qxgeditDrumEg (
-	QWidget *pParent, Qt::WindowFlags wflags )
-	: QFrame(pParent, wflags),
+qxgeditDrumEg::qxgeditDrumEg ( QWidget *pParent )
+	: QFrame(pParent),
 		m_iAttack(0), m_iDecay1(0), m_iDecay2(0),
 		m_poly(6), m_iDragNode(-1)
 {
