@@ -1,7 +1,7 @@
 // qxgeditMainForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2023, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2024, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -1195,10 +1195,10 @@ void qxgeditMainForm::setup ( qxgeditOptions *pOptions )
 	m_iDirtyCount = 0;
 
 	// Is any session pending to be loaded?
-	if (!m_pOptions->sSessionFile.isEmpty()) {
+	if (!m_pOptions->sessionFiles.isEmpty()) {
 		// Just load the prabably startup session...
-		if (loadSessionFile(m_pOptions->sSessionFile))
-			m_pOptions->sSessionFile.clear();
+		if (loadSessionFile(m_pOptions->sessionFiles.first()))
+			m_pOptions->sessionFiles.clear();
 	} else {
 		// Open up with a new empty session...
 		newSession();
@@ -1912,7 +1912,7 @@ void qxgeditMainForm::helpAbout (void)
 	QString sText = "<h1>" QXGEDIT_TITLE "</h1>\n";
 	sText += "<p>" + tr(QXGEDIT_SUBTITLE) + "<br />\n";
 	sText += "<br />\n";
-	sText += tr("Version") + ": <b>" CONFIG_BUILD_VERSION "</b><br />\n";
+	sText += tr("Version") + ": <b>" PROJECT_VERSION "</b><br />\n";
 //	sText += "<small>" + tr("Build") + ": " CONFIG_BUILD_DATE "</small><br />\n";
 #ifdef CONFIG_DEBUG
 	sText += "<small><font color=\"red\">";
@@ -1934,6 +1934,7 @@ void qxgeditMainForm::helpAbout (void)
 	sText += tr("This program is free software; you can redistribute it and/or modify it") + "<br />\n";
 	sText += tr("under the terms of the GNU General Public License version 2 or later.");
 	sText += "</small>";
+	sText += "<br />\n";
 	sText += "</p>\n";
 
 	QMessageBox::about(this, tr("About"), sText);
