@@ -1,7 +1,7 @@
 // qxgeditOptions.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2022, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2024, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -23,6 +23,7 @@
 #include "qxgeditOptions.h"
 
 #include <QWidget>
+#include <QFileInfo>
 #include <QTextStream>
 
 #include <QApplication>
@@ -127,7 +128,7 @@ void qxgeditOptions::saveOptions (void)
 {
 	// Make program version available in the future.
 	m_settings.beginGroup("/Program");
-	m_settings.setValue("/Version", CONFIG_BUILD_VERSION);
+	m_settings.setValue("/Version", PROJECT_VERSION);
 	m_settings.endGroup();
 
 	// And go into general options group.
@@ -227,8 +228,6 @@ void qxgeditOptions::print_usage ( const QString& arg0 )
 // Parse command line arguments into m_settings.
 bool qxgeditOptions::parse_args ( const QStringList& args )
 {
-	int iCmdArgs = 0;
-
 #if QT_VERSION >= QT_VERSION_CHECK(5, 2, 0)
 
 	QCommandLineParser parser;
@@ -242,30 +241,28 @@ bool qxgeditOptions::parse_args ( const QStringList& args )
 		QObject::tr("[session-file]"));
 	parser.process(args);
 
-	foreach(const QString& sArg, parser.positionalArguments()) {
-		if (iCmdArgs > 0)
-			sSessionFile += ' ';
-		sSessionFile += sArg;
-		++iCmdArgs;
+	foreach (const QString& sArg, parser.positionalArguments()) {
+		sessionFiles.append(QFileInfo(sArg).absoluteFilePath());
 	}
 
 #else
 
+	int iCmdArgs = 0;
+
 	QTextStream out(stderr);
 	const QString sEol = "\n\n";
 	const int argc = args.count();
 
 	for (int i = 1; i < argc; ++i) {
 
+		QString sArg = args.at(i);
+
 		if (iCmdArgs > 0) {
-			sSessionFile += ' ';
-			sSessionFile += args.at(i);
+			sessionFiles.append(QFileInfo(sArg).absoluteFilePath());
 			++iCmdArgs;
 			continue;
 		}
 
-		QString sArg = args.at(i);
-
 		if (sArg == "-h" || sArg == "--help") {
 			print_usage(args.at(0));
 			return false;
@@ -278,13 +275,12 @@ bool qxgeditOptions::parse_args ( const QStringList& args )
 			out << '\n';
 			out << QString("%1: %2\n")
 				.arg(QXGEDIT_TITLE)
-				.arg(CONFIG_BUILD_VERSION);
+				.arg(PROJECT_VERSION);
 			return false;
-		}
-		else {
+		} else {
 			// If we don't have one by now,
 			// this will be the startup session file...
-			sSessionFile += sArg;
+			sessionFiles.append(QFileInfo(sArg).absoluteFilePath());
 			++iCmdArgs;
 		}
 	}
